@@ -58,25 +58,6 @@ draw_atom(int si)
   
   static GC depth_one_gc;
   
-  static Pixmap stipple_pm;
-  
-  if(!stipple_pm){
-    
-    stipple_pm=XCreatePixmap
-      (display,root_window,2,2,1);
-    
-    depth_one_gc=XCreateGC(display,stipple_pm,0,0);
-    
-    XSetForeground(display,depth_one_gc,BlackPixelOfScreen(screen_ptr));
-    XFillRectangle(display,stipple_pm,depth_one_gc,0,0,2,2);
-    
-    XSetForeground(display,depth_one_gc,WhitePixelOfScreen(screen_ptr));
-    XDrawPoint(display,stipple_pm,depth_one_gc,1,1);
-    XDrawPoint(display,stipple_pm,depth_one_gc,2,2);
-    
-    XSetStipple(display,gc,stipple_pm);
-  }
-  
   radius = get_atom_radius(si);
   
   gcv.foreground=atoms[si].pixel;
@@ -106,41 +87,21 @@ draw_atom(int si)
        0,
        360*64);
     
-    if((atoms[si].sel == 1))
+    if(atoms[si].sel == 1)
       {
-        /* stipple */
-        gcv.foreground = BlackPixelOfScreen(screen_ptr);
-        gcv.fill_style = FillStippled;
-      
-        XChangeGC(display,
-                  gc,
-                  GCForeground | GCFillStyle,
-                  &gcv);
-      
-        XFillArc (display,
-                  canvas_pm,
-                  gc,
-                  (int) (canvas_coord[0] - radius),
-                  (int) (canvas_coord[1] - radius),
-                  (int) (2.0 * radius),
-                  (int) (2.0 * radius),
-                  0,
-                  360*64);
-
-        gcv.fill_style = FillSolid;
-        
-        XChangeGC(display,
-                  gc,
-                  GCFillStyle,
-                  &gcv);
-
       }
   }
   
-  gcv.fill_style=FillSolid;
-  gcv.foreground=BlackPixelOfScreen(screen_ptr);
+  if (atoms[si].sel == 1)
+    {
+      gcv.foreground = sel_color.pixel;
+    }
+  else
+    {
+      gcv.foreground = BlackPixelOfScreen (screen_ptr);
+    }
 
-  XChangeGC(display,gc,GCFillStyle|GCForeground,&gcv);
+  XChangeGC (display, gc, GCForeground, &gcv);
   
   XDrawArc
     (display,
@@ -181,12 +142,12 @@ draw_bond(int si, int sj)
 {
 
   double get_atom_radius(int);
-  void update_canvas_bond_points(int, int, Boolean);
+  void update_canvas_bond_points(int, int, Boolean, double);
   
   
   XPoint bond_points[5];
   
-  update_canvas_bond_points(si, sj, 0);
+  update_canvas_bond_points(si, sj, 0, 0.0);
     
   gcv.line_width=0;
   gcv.foreground=atoms[si].pixel;
@@ -218,22 +179,6 @@ draw_bond(int si, int sj)
   gcv.foreground=BlackPixelOfScreen(screen_ptr);
   XChangeGC(display,gc,GCForeground,&gcv);
     
-  if(atoms[si].sel){ /* Add stippled overlay */
-    gcv.fill_style=FillStippled;
-    XChangeGC(display,gc,GCFillStyle,&gcv);
-
-    XFillPolygon(display,
-                 canvas_pm,
-                 gc,
-                 bond_points,
-                 4,
-                 Complex,
-                 CoordModeOrigin);
-
-    gcv.fill_style=FillSolid;
-    XChangeGC(display,gc,GCFillStyle,&gcv);
-  }
-    
   gcv.foreground=atoms[sj].pixel;
   XChangeGC(display,gc,GCForeground,&gcv);
 
@@ -262,22 +207,6 @@ draw_bond(int si, int sj)
   gcv.foreground=BlackPixelOfScreen(screen_ptr);
   XChangeGC(display,gc,GCForeground,&gcv);
     
-  if(atoms[sj].sel){ /* Add stippled overlay */
-    gcv.fill_style=FillStippled;
-    XChangeGC(display,gc,GCFillStyle,&gcv);
-      
-    XFillPolygon(display,
-                 canvas_pm,
-                 gc,
-                 bond_points,
-                 4,
-                 Complex,
-                 CoordModeOrigin);
-      
-    gcv.fill_style=FillSolid;
-    XChangeGC(display,gc,GCFillStyle,&gcv);
-  }
-
   /* Draw the lines 0-2-3-5-0 */
 
   bond_points[0].x = canvas_bond_points[0].x;
@@ -723,7 +652,7 @@ get_atom_radius(int i)
   /* Scale by covalent of van der Waals radius, depending on whether
      the atom is bonded to anything else */
 
-  if (bond_adjacency_list[i] == NULL)
+  if ((bond_adjacency_list[i] == NULL) && (never_use_vdw == 0))
     {
       radius *= atoms[i].vdw_rad;
     }
@@ -775,7 +704,7 @@ get_atom_radius(int i)
    is covered using the Boolean variable ps_corr */
 
 void
-update_canvas_bond_points(int si, int sj, Boolean ps_corr)
+update_canvas_bond_points(int si, int sj, Boolean ps_corr, double offset)
 {
 
   void convert_to_canvas_coords(double *, double *, Boolean);
@@ -835,8 +764,17 @@ update_canvas_bond_points(int si, int sj, Boolean ps_corr)
   
   for(i=0; i < 3; i++)
     {
-      l[i] = a[i] + (atoms[si].cov_rad * local_at_scale * r[i]);
-      m[i] = a[i] + a_prop * mod_r * r[i];
+      if (local_at_scale != 0)
+        {
+          l[i] = a[i] +
+            (atoms[si].cov_rad * local_at_scale + offset) * r[i];
+        }
+      else
+        {
+          l[i] = a[i] + (offset * r[i]);
+        }
+
+      m[i] = a[i] + a_prop * (mod_r - offset) * r[i];
       n[i] = b[i] - (atoms[sj].cov_rad * local_at_scale * r[i]);
     }
 
@@ -848,8 +786,14 @@ update_canvas_bond_points(int si, int sj, Boolean ps_corr)
   
   for(i = 0; i < 3; i++)
     {
-      
-      half_bond_width = (sb_wid * p[i] / 2.0);
+      if ((i == 0) || (i == 1))
+        {
+          half_bond_width = ((sb_wid - (2.0 * offset)) * p[i] / 2.0);
+        }
+      else
+        {
+          half_bond_width = (sb_wid * p[i] / 2.0);
+        }
 
       cartesian_bond_points[0][i] = l[i] - half_bond_width;
       cartesian_bond_points[1][i] = m[i] - half_bond_width;
@@ -873,7 +817,6 @@ update_canvas_bond_points(int si, int sj, Boolean ps_corr)
       canvas_bond_points[i].y = xy_temp[1];
           
     }
-      
 }
 
 
@@ -928,22 +871,22 @@ update_canvas_hbond_points(int si, int sj, Boolean ps_corr)
 
       /* If bonded use covalent radius, otherwise van der Waals */
 
-      if (bond_adjacency_list[si] != NULL)
-        {
-          radius[0] = atoms[si].cov_rad;
-        }
-      else
+      if ((bond_adjacency_list[si] == NULL) && (never_use_vdw == 0))
         {
           radius[0] = atoms[si].vdw_rad;
         }
-
-      if (bond_adjacency_list[sj] != NULL)
+      else
         {
-          radius[1] = atoms[sj].cov_rad;
+          radius[0] = atoms[si].cov_rad;
+        }
+
+      if ((bond_adjacency_list[sj] == NULL) && (never_use_vdw == 0))
+        {
+          radius[1] = atoms[sj].vdw_rad;
         }
       else
         {
-          radius[1] = atoms[sj].vdw_rad;
+          radius[1] = atoms[sj].cov_rad;
         }
 
 
