@@ -26,6 +26,8 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "config.h"
 #endif
 
+#ifdef GL
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
@@ -35,7 +37,6 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include <X11/extensions/SGIStereo.h> /* aro */
 #endif /* SGI_STEREO */
 
-#ifdef GL
 #include "bbox.h"
 #include "bonds.h"
 #include "defs.h"
@@ -48,6 +49,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 /* Function prototypes */
 
 double get_atom_scale (void);
+double get_bond_scale (void);
 double get_bond_width (void);
 double get_hbond_scale (void);
 double gl_get_rendered_atom_size (int);
@@ -72,7 +74,7 @@ static int gl_copy_canvas = 0;
 static int gl_initialized = 0;
 static int gl_no_atom_segments = 12;
 static int gl_no_bond_segments = 8;
-static int gl_use_lighting = 1;
+static int gl_use_lighting = 0;
 static int gl_use_specular_lighting = 0;
 static int gl_sep = 30; /* aro */
 static GLfloat gl_shininess[] = {64};
@@ -82,36 +84,10 @@ static double gl_fov = 50;
 static double gl_max_z, gl_min_z;
 static double gl_perspective_near = 0, gl_perspective_far = 0;
 
-static GLubyte measure_stipple[] = {
-  0xAA, 0xAA, 0xAA, 0xAA, 0x00, 0x00, 0x00, 0x00,
-  0xAA, 0xAA, 0xAA, 0xAA, 0x00, 0x00, 0x00, 0x00,
-  0xAA, 0xAA, 0xAA, 0xAA, 0x00, 0x00, 0x00, 0x00,
-  0xAA, 0xAA, 0xAA, 0xAA, 0x00, 0x00, 0x00, 0x00,
-  0xAA, 0xAA, 0xAA, 0xAA, 0x00, 0x00, 0x00, 0x00,
-  0xAA, 0xAA, 0xAA, 0xAA, 0x00, 0x00, 0x00, 0x00,
-  0xAA, 0xAA, 0xAA, 0xAA, 0x00, 0x00, 0x00, 0x00,
-  0xAA, 0xAA, 0xAA, 0xAA, 0x00, 0x00, 0x00, 0x00,
-  0xAA, 0xAA, 0xAA, 0xAA, 0x00, 0x00, 0x00, 0x00,
-  0xAA, 0xAA, 0xAA, 0xAA, 0x00, 0x00, 0x00, 0x00,
-  0xAA, 0xAA, 0xAA, 0xAA, 0x00, 0x00, 0x00, 0x00,
-  0xAA, 0xAA, 0xAA, 0xAA, 0x00, 0x00, 0x00, 0x00,
-  0xAA, 0xAA, 0xAA, 0xAA, 0x00, 0x00, 0x00, 0x00,
-  0xAA, 0xAA, 0xAA, 0xAA, 0x00, 0x00, 0x00, 0x00,
-  0xAA, 0xAA, 0xAA, 0xAA, 0x00, 0x00, 0x00, 0x00,
-  0xAA, 0xAA, 0xAA, 0xAA, 0x00, 0x00, 0x00, 0x00,
-};
-
 static GLUquadricObj *gl_quadric = NULL;
 
-/* enum types */
-
-enum gl_render_types
-{
-  BALL_AND_STICK,
-  TUBES
-};
-
 static enum gl_render_types gl_render_type = BALL_AND_STICK;
+static enum gl_render_types gl_current_render_type;
 
 void gl_render (void)
 {
@@ -155,10 +131,10 @@ void gl_render (void)
   height_scale = width_scale * (canvas_width / (float) canvas_height);    
   glScalef (width_scale, height_scale, 1.0f);
 
-  gluLookAt (0.0, 0.0, gl_eye, 0.0, 0.0, -1.0, 0.0, 1.0, 0.0);
+  gluLookAt (0.0, 0.0, gl_eye, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
 
   /* aro--> */
-  if(get_gl_render_stereo())
+  if(get_gl_render_stereo() && depth_is_on)
     {
       /* Calculate offset between eyes.  Max distance between eyes should be 
          (distance to projection plane) / 15 */
@@ -314,7 +290,7 @@ void gl_render (void)
     }
 
   /* aro - change color mask so all of RGB color fields are cleared */
-  if(get_gl_render_stereo() == RED_BLUE) 
+  if((get_gl_render_stereo() == RED_BLUE) && depth_is_on)
     glColorMask(GL_TRUE,GL_TRUE,GL_TRUE,GL_TRUE);
   
   /* Note that we are drawing onto the canvas directly, since I
@@ -397,10 +373,14 @@ void gl_render_atoms (void)
   /* Return if atoms not being display and in ball and stick mode;
      note that we draw atoms if tubes mode if the bond is visible */
 
-  if ((atom_flag == 0) && (gl_render_type == BALL_AND_STICK)) return;
-
   for (i = 0; i < no_atoms; i++)
     {
+      gl_current_render_type = (atoms[i].gl_render_type == DEFAULT) ?
+        get_gl_render_type () : atoms[i].gl_render_type;
+        
+      if ((atom_flag == 0) &&
+          ((gl_current_render_type == BALL_AND_STICK) ||
+           ((gl_current_render_type == TUBES) && (! gl_use_lighting)))) return;
 
       if ((atoms[i].visi == 1) || (view_ghost_atoms == 1))
         {
@@ -411,8 +391,14 @@ void gl_render_atoms (void)
 
 void gl_render_atom (int i)
 {
+  int j;
+
+  double get_smallest_cov_rad (void);
+  double line_width, atom_size;
+
+  char string[16]="", sub_string[8]="", *labels = "ABCD";
+
   enum render_stereo_types get_gl_render_stereo(void);/* aro */
-  double atom_size;
   GLfloat red, green, blue;
 
   /* For tubes mode, we don't draw an atom (a) when bonds are not
@@ -420,7 +406,7 @@ void gl_render_atom (int i)
      displayed and the atom is not bonded. In essense, atoms and bonds
      are mutually exclusive. */
 
-  if (gl_render_type == TUBES)
+  if ((gl_current_render_type == TUBES) && gl_use_lighting)
     {
       if (((bond_flag == 0) && (bond_adjacency_list[i] != NULL)) ||
           ((atom_flag == 0) && (bond_adjacency_list[i] == NULL))) return;
@@ -432,7 +418,7 @@ void gl_render_atom (int i)
 
   /* aro - We need to convert the colors to grayscale in order for 
      red and blue color masks to work properly */
-  if(get_gl_render_stereo() == RED_BLUE)
+  if((get_gl_render_stereo() == RED_BLUE) && depth_is_on)
     {
       red = (red + blue + green) / 3;
       blue = red;
@@ -442,29 +428,6 @@ void gl_render_atom (int i)
   glPushMatrix ();
   glTranslatef (atoms[i].x, atoms[i].y, atoms[i].z);
 
-  /* If the atom is selected, draw a black disk behind the atom and
-     stipple the atom itself */
-   
-  if(atoms[i].sel == 1)
-    {
-
-      glPolygonStipple (measure_stipple);
-      glColor3f (0.0, 0.0, 0.0);
-
-/*        gluDisk (quadric, */
-/*                 0.0, */
-/*                 atom_scale * atoms[i].cov_rad, */
-/*                 gl_no_atom_segments, */
-/*                 1); */
-
-/*        gluSphere (quadric, */
-/*                   atom_scale * atoms[i].cov_rad, */
-/*                   gl_no_atom_segments, */
-/*                   gl_no_atom_segments); */
-
-      glEnable (GL_POLYGON_STIPPLE);
-    }
-
   if ((! mouse_motion_p ()) || (! outline_mode_p ()))
     {
       glColor3f (red, green, blue);
@@ -474,41 +437,13 @@ void gl_render_atom (int i)
       glColor3f (0.0, 0.0, 0.0);
     }
 
-  if (gl_render_type == BALL_AND_STICK)
-    {
-      if (bond_adjacency_list[i] == NULL)
-        {
-          /* If no bond, use van der Waals radius  */
-          atom_size = get_atom_scale () * atoms[i].vdw_rad;
-        }
-      else
-        {
-          atom_size = get_atom_scale () * atoms[i].cov_rad;
-        }
-    }
-  else if (gl_render_type == TUBES)
-    {
-      /* If we are not bonded to any other atoms, use normal size */
-
-      if (bond_adjacency_list[i] == NULL)
-        {
-          atom_size = get_atom_scale () * atoms[i].vdw_rad;
-        }
-      else
-        {
-          /* We're getting the diameter, and we want the radius */
-          atom_size = get_bond_width () / 2.0;
-        }
-    }
-  else
-    {
-      atom_size = 0.0;
-    }
+  atom_size = gl_get_rendered_atom_size (i);
+  line_width = 0.1 * get_atom_scale () * get_smallest_cov_rad ();
 
   /* Set up rendering for elliptical atoms */
 
-  if (((gl_render_type == BALL_AND_STICK) ||
-       ((gl_render_type == TUBES) && bond_adjacency_list[i] == NULL)) &&
+  if (((gl_current_render_type == BALL_AND_STICK) ||
+       ((gl_current_render_type == TUBES) && bond_adjacency_list[i] == NULL)) &&
       (atoms[i].is_ellipse == 1))
     {
       void euler_to_matrix (double *, double *);
@@ -556,10 +491,66 @@ void gl_render_atom (int i)
 
   if ((! mouse_motion_p ()) || (! outline_mode_p ()))
     {
-      gluSphere (gl_quadric,
-                 atom_size,
-                 gl_no_atom_segments,
-                 gl_no_atom_segments);
+      /* Draw border for selected atoms */
+
+      if (atoms[i].sel == 1)
+        {
+          glColor3f (1.0, 0.65, 0.0); /* Orange */
+
+          glDisable (GL_LIGHTING);
+
+          gluDisk (gl_quadric,
+                   atom_size - line_width,
+                   atom_size,
+                   gl_no_atom_segments,
+                   1);
+
+          if (gl_use_lighting) glEnable (GL_LIGHTING);
+
+          glColor3f (red, green, blue);
+        }
+
+      if (gl_use_lighting)
+        {
+          double size;
+
+          if (atoms[i].sel == 0)
+            {
+              size = atom_size;
+            }
+          else
+            {
+              size = atom_size - line_width;
+            }
+
+          gluSphere (gl_quadric,
+                     size,
+                     gl_no_atom_segments,
+                     gl_no_atom_segments);
+        }
+      else
+        {
+          gluDisk (gl_quadric,
+                   0,
+                   atom_size,
+                   gl_no_atom_segments,
+                   1);
+
+          if (atoms[i].sel == 0)
+            {
+              glColor3f (0.0, 0.0, 0.0);
+
+              /* Move slightly forward */
+
+              glTranslatef (0.0, 0.0, 0.01);
+
+              gluDisk (gl_quadric,
+                       atom_size - line_width,
+                       atom_size,
+                       gl_no_atom_segments,
+                       1);
+            }
+        }
     }
   else
     {
@@ -570,10 +561,50 @@ void gl_render_atom (int i)
                1);              /* Number of rings */
     }
 
+  if (atom_flag == 1)
+    {
+      /* Atom numbers */
+
+      if (at_nos_flag)
+        {
+          sprintf (string, "%d ", i + 1);
+        }
+
+      /* Atom symbols */
+
+      if (at_sym_flag)
+        {
+          sprintf (sub_string, "%s ", atoms[i].label);
+          strcat (string, sub_string);
+        }
+
+      /* Atom selections */
+
+      for (j = 0; j < 4; j++)
+        {
+          if(i == selected[j])
+            {
+              sprintf (sub_string, "%c ", labels[j]);
+              strcat (string, sub_string);
+            }
+        }
+
+      if (strlen (string) > 0)
+        {
+          glColor3f (0.0, 0.0, 0.0);
+          glRasterPos3f (0, 0, atom_size * 1.1);
+
+          for (j = 0; j < strlen (string); j++)
+            {
+              glutBitmapCharacter (GLUT_BITMAP_HELVETICA_12, string[j]);
+            }
+
+          glRasterPos3f (0, 0, atom_size * 1.1);
+          gl2psText(string, "Helvetica", 12);
+        }
+    }
+
   glPopMatrix ();
-
-  glDisable (GL_POLYGON_STIPPLE);
-
 }
 
 
@@ -581,25 +612,25 @@ void gl_render_bbox (void)
 {
   glColor3f (0.0, 0.0, 0.0);
 
-  gl_render_bbox_line(0, 1);
-  gl_render_bbox_line(0, 2);
-  gl_render_bbox_line(0, 4);
-  gl_render_bbox_line(1, 3);
-  gl_render_bbox_line(1, 5);
-  gl_render_bbox_line(2, 3);
-  gl_render_bbox_line(2, 6);
-  gl_render_bbox_line(3, 7);
-  gl_render_bbox_line(4, 5);
-  gl_render_bbox_line(4, 6);
-  gl_render_bbox_line(5, 7);
-  gl_render_bbox_line(6, 7);
+  gl_render_bbox_line (0, 1);
+  gl_render_bbox_line (0, 2);
+  gl_render_bbox_line (0, 4);
+  gl_render_bbox_line (1, 3);
+  gl_render_bbox_line (1, 5);
+  gl_render_bbox_line (2, 3);
+  gl_render_bbox_line (2, 6);
+  gl_render_bbox_line (3, 7);
+  gl_render_bbox_line (4, 5);
+  gl_render_bbox_line (4, 6);
+  gl_render_bbox_line (5, 7);
+  gl_render_bbox_line (6, 7);
 
-  gl_render_bbox_face (0, 1, 3, 2);
-  gl_render_bbox_face (4, 5, 7, 6);
-  gl_render_bbox_face (0, 2, 6, 4);
-  gl_render_bbox_face (1, 3, 7, 5);
-  gl_render_bbox_face (0, 1, 5, 4);
-  gl_render_bbox_face (2, 3, 7, 6);
+/*   gl_render_bbox_face (0, 1, 3, 2); */
+/*   gl_render_bbox_face (4, 5, 7, 6); */
+/*   gl_render_bbox_face (0, 2, 6, 4); */
+/*   gl_render_bbox_face (1, 3, 7, 5); */
+/*   gl_render_bbox_face (0, 1, 5, 4); */
+/*   gl_render_bbox_face (2, 3, 7, 6); */
 }
 
 
@@ -664,8 +695,11 @@ void gl_render_bond (int i,
                      int j)
 {
   enum render_stereo_types get_gl_render_stereo(void);/* aro */
+  double get_smallest_cov_rad (void);
+  void update_canvas_bond_points (int, int, Boolean, double);
 
-  double angle, b[3], bond_radius, length, mod_b;
+  int k;
+  double angle, b[3], bond_radius, length, mod_b, line_width;
   GLfloat red, green, blue;
 
   /* We want the radius, ie half the width which represents the
@@ -690,7 +724,7 @@ void gl_render_bond (int i,
 
   /* aro - We need to convert the colors to grayscale in order for 
      red and blue color masks to work properly */
-  if(get_gl_render_stereo() == RED_BLUE)
+  if((get_gl_render_stereo() == RED_BLUE) && depth_is_on)
     {
       red = (red + blue + green) / 3;
       blue = red;
@@ -700,27 +734,108 @@ void gl_render_bond (int i,
   glColorMaterial (GL_FRONT, GL_DIFFUSE);
   glColor3f (red, green, blue);
 
-  glPushMatrix ();
+  if (gl_use_lighting)
+    {
+      glPushMatrix ();
 
-  glTranslatef (atoms[i].x, atoms[i].y, atoms[i].z);
+      glTranslatef (atoms[i].x, atoms[i].y, atoms[i].z);
 
-  glRotatef (180.0, 0.0, 0.0, 1.0);
-  glRotatef (angle, b[1], -1.0 * b[0], 0.0);
+      glRotatef (180.0, 0.0, 0.0, 1.0);
+      glRotatef (angle, b[1], -1.0 * b[0], 0.0);
 
-  /* length of bond segment */
+      /* length of bond segment */
 
-  length = (mod_b * atoms[i].cov_rad /
+      length = (mod_b * atoms[i].cov_rad /
             (atoms[i].cov_rad + atoms[j].cov_rad));
+      
+      gluCylinder (gl_quadric,
+                   bond_radius,
+                   bond_radius,
+                   length,
+                   gl_no_bond_segments,
+                   1);
 
-  gluCylinder (gl_quadric,
-               bond_radius,
-               bond_radius,
-               length,
-               gl_no_bond_segments,
-               1);
+      glPopMatrix ();
+    }
+  else
+    {
+      /* Outline */
 
-  glPopMatrix ();
+      update_canvas_bond_points (i, j, False, 0.0);
 
+      glColor3f (0.0, 0.0, 0.0);
+
+      glPushMatrix ();
+
+      glTranslatef (0.0, 0.0, -0.01);
+
+      glBegin (GL_POLYGON);
+
+      k = 0;
+
+      glVertex3f (cartesian_bond_points[k][0],
+                  cartesian_bond_points[k][1],
+                  cartesian_bond_points[k][2]);
+
+      k = 1;
+
+      glVertex3f (cartesian_bond_points[k][0],
+                  cartesian_bond_points[k][1],
+                  cartesian_bond_points[k][2]);
+
+      k = 4;
+
+      glVertex3f (cartesian_bond_points[k][0],
+                  cartesian_bond_points[k][1],
+                  cartesian_bond_points[k][2]);
+
+      k = 5;
+
+      glVertex3f (cartesian_bond_points[k][0],
+                  cartesian_bond_points[k][1],
+                  cartesian_bond_points[k][2]);
+
+      glEnd ();
+
+      glPopMatrix ();
+
+      /* Colour blocks */
+
+      glColor3f (red, green, blue);
+
+      line_width = 0.1 * get_bond_scale () * get_smallest_cov_rad ();
+      
+      update_canvas_bond_points (i, j, False, line_width);
+
+      glBegin (GL_POLYGON);
+
+      k = 0;
+
+      glVertex3f (cartesian_bond_points[k][0],
+                  cartesian_bond_points[k][1],
+                  cartesian_bond_points[k][2]);
+
+      k = 1;
+
+      glVertex3f (cartesian_bond_points[k][0],
+                  cartesian_bond_points[k][1],
+                  cartesian_bond_points[k][2]);
+
+      k = 4;
+
+      glVertex3f (cartesian_bond_points[k][0],
+                  cartesian_bond_points[k][1],
+                  cartesian_bond_points[k][2]);
+
+      k = 5;
+
+      glVertex3f (cartesian_bond_points[k][0],
+                  cartesian_bond_points[k][1],
+                  cartesian_bond_points[k][2]);
+
+      glEnd ();
+
+    }
 }
 
 void gl_render_hbonds (void)
@@ -841,13 +956,13 @@ double gl_get_rendered_atom_size (int i)
      account the drawing mode (BALL_AND_STICK or TUBES) and whether or
      not atoms and bonds are being displayed */
 
-  if (gl_render_type == BALL_AND_STICK)
+  if ((gl_current_render_type == BALL_AND_STICK) || (! gl_use_lighting))
     {
       if (atom_flag)
         {
           /* If not bonded use VDW, otherwise covalent radius */
 
-          if (bond_adjacency_list[i] == NULL)
+          if ((bond_adjacency_list[i] == NULL) && (never_use_vdw == 0))
             {
               atom_size = get_atom_scale () * atoms[i].vdw_rad;
             }
@@ -861,7 +976,7 @@ double gl_get_rendered_atom_size (int i)
           atom_size = 0;        /* Atoms not being rendered */
         }
     }
-  else if (gl_render_type == TUBES)
+  else if (gl_current_render_type == TUBES)
     {
       if (bond_adjacency_list[i] == NULL)
         {
@@ -869,7 +984,14 @@ double gl_get_rendered_atom_size (int i)
 
           if (atom_flag)
             {
-              atom_size = get_atom_scale () * atoms[i].vdw_rad;
+              if (never_use_vdw == 1)
+                {
+                  atom_size = get_atom_scale () * atoms[i].cov_rad;
+                }
+              else
+                {
+                  atom_size = get_atom_scale () * atoms[i].vdw_rad;
+                }
             }
           else
             {
@@ -908,7 +1030,6 @@ void gl_render_vectors (void)
     }
 
   glColorMaterial (GL_FRONT, GL_DIFFUSE);
-  glColor3f (0.5, 0.5, 0.5);
 
   for (i = 0; i < no_atoms; i++)
     {
@@ -932,12 +1053,14 @@ void gl_render_vector (int i, int j)
   double angle, height, mod_of_vector, top_radius;
   double vector_arrow_angle, vector_arrow_scale, vector_scale;
   double vector[3];
+  double atom_size = gl_get_rendered_atom_size (i);
 
   vector_arrow_angle = get_vector_arrow_angle ();
   vector_arrow_scale = get_vector_arrow_scale ();
   vector_scale = get_vector_scale ();
- 
+
   /* Draw stem */
+  glColor3f (0.5, 0.5, 0.5); 
 
   vector[0] = atoms[i].v[j][0] * vector_scale;
   vector[1] = atoms[i].v[j][1] * vector_scale;
@@ -957,10 +1080,12 @@ void gl_render_vector (int i, int j)
   glRotatef (180.0, 0.0, 0.0, 1.0);
   glRotatef (angle, vector[1], -1.0 * vector[0], 0.0);
 
+  glTranslatef (0.0, 0.0, atom_size);
+
   gluCylinder (gl_quadric,
                0.025,
                0.025,
-               mod_of_vector - height,
+               mod_of_vector - (height + atom_size),
                16,
                1);
 
@@ -968,6 +1093,8 @@ void gl_render_vector (int i, int j)
 
   /* Draw head  */
 
+  glColor3f (0.0, 0.0, 0.0);
+
   top_radius = height *
     tan (vector_arrow_angle * DEG2RAD);
 
@@ -1049,8 +1176,23 @@ void gl_set_perspective (void)
   glLoadIdentity ();
 
   gl_update_near_and_far ();
-  gluPerspective (gl_fov, 1.0, gl_perspective_near, gl_perspective_far);
 
+  if (depth_is_on)
+    {
+      gluPerspective (gl_fov, 1.0, gl_perspective_near, gl_perspective_far);
+    }
+  else
+    {
+      double h;
+
+      /* Use the field of view angle (gl_fov) to get the dimensions of
+         the rendered scene (h) at the origin; this is a square gl_eye
+         from the viewpoint */
+
+      h = gl_eye * tan (gl_fov * PI / 360.0);
+
+      glOrtho (-h, h, -h, h, gl_perspective_near, gl_perspective_far);
+    }
 }
 
 void gl_update_near_and_far (void)
@@ -1226,7 +1368,7 @@ void set_gl_sep (int new_value)
 
 /* Logical variable used to determine if GL rendering is active */
 
-static Boolean render_using_gl = False;
+static Boolean render_using_gl = True;
 
 void set_render_using_gl (Boolean value)
 {
@@ -1243,9 +1385,9 @@ int get_gl_render_type (void)
   return (gl_render_type);
 }
 
-void set_gl_render_type (int new_value)
+void set_gl_render_type (enum gl_render_types new_value)
 {
-  gl_render_type = (enum gl_render_types) new_value;
+  gl_render_type = new_value;
 }
 
 /* aro--> */
