@@ -180,6 +180,7 @@ Widget edit_atbd_sizes_dialog = NULL;
 Widget edit_bond_fudges_dialog = NULL;
 Widget edit_element_props_dialog = NULL; /* aro */
 Widget gl_render_dialog = NULL; /* aro */
+Widget scale_coords_dialog = NULL;
 
 Widget *meas_label_w=NULL;
 Widget nth_menu[6];
@@ -258,6 +259,7 @@ extern Widget edit_atbd_sizes_dialog;
 extern Widget edit_bond_fudges_dialog;
 extern Widget edit_element_props_dialog;/* aro */
 extern Widget gl_render_dialog; /* aro */
+extern Widget scale_coords_dialog;
 
 extern Widget *meas_label_w;
 extern Widget nth_menu[6];
