@@ -43,7 +43,7 @@ make_menus(Widget main_w, Widget menubar)
 
   XmString file,open,revert,save,merge,export,print,quit;/* aro */       /* 0th menu */
   XmString cont,fram,anim,meas,pers;                   /* 1st */
-  XmString edit,visi,posn,ab_sz,bd_fd,vdisp,elem;   /* 2nd */
+  XmString edit,visi,posn,sc_co,ab_sz,bd_fd,vdisp,elem;   /* 2nd */
   XmString track,tr_rotl,tr_roto,cent,tr_reor,tr_repo,
     reflect_x,reflect_y,reflect_z,invert;              /* 3rd */
   XmString view,atoms,bonds,hbonds,vects,at_nos;     /* 4th */
@@ -161,6 +161,7 @@ make_menus(Widget main_w, Widget menubar)
 
   visi=XmStringCreateLocalized("Visible...");
   posn=XmStringCreateLocalized("Positions...");
+  sc_co=XmStringCreateLocalized("Scale coordinates...");
   ab_sz=XmStringCreateLocalized("Atom/bond sizes...");
   bd_fd=XmStringCreateLocalized("Bond factors...");
   vdisp = XmStringCreateLocalized("Vector display...");
@@ -177,6 +178,7 @@ make_menus(Widget main_w, Widget menubar)
     (menubar, "edit_menu", 2, edit_cb,
      XmVaPUSHBUTTON, visi,  'V', "Ctrl<Key>V", accel1,
      XmVaPUSHBUTTON, posn,  'P', NULL, NULL,
+     XmVaPUSHBUTTON, sc_co, 'S', NULL, NULL,
      XmVaSEPARATOR,
      XmVaPUSHBUTTON, ab_sz, 'A', "Ctrl<Key>A", accel2,
      XmVaPUSHBUTTON, bd_fd, 'B', "Ctrl<Key>B", accel3,
@@ -190,6 +192,7 @@ make_menus(Widget main_w, Widget menubar)
 
   XmStringFree(visi);
   XmStringFree(posn);
+  XmStringFree(sc_co);
   XmStringFree(ab_sz);
   XmStringFree(bd_fd);
   XmStringFree(vdisp);
