@@ -369,6 +369,7 @@ select_atom_for_measure (void)
 {
   void update_selected(int);
 
+  char message[64];
   int closest;
 
   if (no_atoms == 0) return;
@@ -377,7 +378,21 @@ select_atom_for_measure (void)
 
   closest = select_atom_internal (1);
   update_selected (closest);
-    
+
+  if (atoms[closest].sel == 1)
+    {
+      sprintf (message, "Atom %d selected: x = %9.4f, y = %9.4f, z = %9.4f",
+               closest + 1,
+               atoms[closest].x,
+               atoms[closest].y,
+               atoms[closest].z);
+
+      echo_to_message_area (message);
+    }
+  else
+    {
+      clear_message_area ();
+    }
 }
 
 
